@@ -1,3 +1,5 @@
+if test "x$SETX" = x1 ; then set -x ; fi
+
 if test $# = 0 ; then
 TEST=1
 else
@@ -21,7 +23,8 @@ cd ${srcdir}/cdltestfiles; CDLTESTFILES=`pwd` ; cd ${WD}
 cd ${srcdir}/baseline; BASELINE=`pwd` ; cd ${WD}
 cd ${srcdir}/baselineraw; BASELINERAW=`pwd` ; cd ${WD}
 cd ${srcdir}/baselineremote; BASELINEREM=`pwd` ; cd ${WD}
-BASELINEH=${BASELINEREM}
+cd ${srcdir}/baselinehyrax; BASELINEH=`pwd` ; cd ${WD}
+cd ${builddir}/baselinethredds; BASELINETH=`pwd` ; cd ${WD}
 
 setresultdir() {
 rm -fr ${builddir}/$1
@@ -52,6 +55,23 @@ filesexist() {
     done
 }
 
+urlbasename() {
+    constraint=`echo "$1" | cut -d '?' -f2`
+    unconstrained=`echo "$1" | cut -d '?' -f1`
+    base=`basename $unconstrained`
+    prefix=`dirname $unconstrained`
+}
+
+makeurl() {
+    urlbasename "$2"
+    if test "x$constraint" = "x$unconstrained" ; then
+        URL="$1/${prefix}/${base}${FRAG}"
+    else
+	URL="$1/${prefix}/${base}?$constraint${FRAG}"
+    fi
+}
+
+
 finish() {
 if test "x$FAILURES" = x1 ; then
 echo "*** Fail"
