@@ -61,6 +61,7 @@ setup(int tdmr, int argc, char** argv)
     argc--; argv++;
     int expected = 0;
     NCD4mode mode = 0;
+    NCD4INFO* controller = NULL;
 
     switch(tdmr) {
     case TDMR_PARSE:
@@ -96,21 +97,19 @@ setup(int tdmr, int argc, char** argv)
     NCD4_dumpbytes(ncbyteslength(input),ncbytescontents(input),0);
 #endif
 
-    if((metadata=NCD4_newmeta(ncbyteslength(input),ncbytescontents(input)))==NULL)
+    /* Create a fake NCD4INFO */
+    controller = (NCD4INFO*)calloc(1,sizeof(NCD4INFO));
+    if(controller == NULL)
+	fail(NC_ENOMEM);
+    controller->controls.translation = NCD4_TRANSNC4;
+    if(translatenc4)
+	controller->controls.translation = NCD4_TRANSNC4;
+    NCD4_applyclientparamcontrols(controller);
+    if((metadata=NCD4_newmeta(controller))==NULL)
 	fail(NC_ENOMEM);
     metadata->mode = mode;
+    NCD4_attachraw(metadata, ncbyteslength(input),ncbytescontents(input));
 
-    /* Create a fake NCD4INFO */
-    {
-	NCD4INFO* controller = (NCD4INFO*)calloc(1,sizeof(NCD4INFO));
-	if(controller == NULL)
-	    fail(NC_ENOMEM);
-        metadata->controller = controller;
-	controller->controls.translation = NCD4_TRANSNC4;
-        if(translatenc4)
-	    controller->controls.translation = NCD4_TRANSNC4;
-	NCD4_applyclientparamcontrols(controller);
-    }
     if((ret=NCD4_dechunk(metadata))) /* ok for mode == DMR or mode == DAP */
 	fail(ret);
 #ifdef DEBUG
@@ -163,7 +162,6 @@ cleanup(int ret)
     if(metadata->controller != NULL)
 	free(metadata->controller);
     NCD4_reclaimMeta(metadata);
-    ncbytesfree(input);
     ncbytesfree(output);
     if(ret)
 	fail(ret);
