@@ -60,6 +60,7 @@ static const struct KEYWORDINFO {
 {"UInt64", NCD4_VAR,NC_UINT64,NULL},
 {"UInt8", NCD4_VAR,NC_UBYTE,NULL},
 {"URL", NCD4_VAR,NC_STRING,"String"},
+{"Url", NCD4_VAR,NC_STRING,"String"},
 };
 typedef struct KEYWORDINFO KEYWORDINFO;
 
@@ -67,6 +68,7 @@ static const struct ATOMICTYPEINFO {
     char* name; nc_type type; size_t size;
 } atomictypeinfo[] = {
 /* Keep in sorted order for binary search */
+/* Use lower case for canonical comparison, but keep proper name here */
 {"Byte",NC_BYTE,sizeof(char)},
 {"Char",NC_CHAR,sizeof(char)},
 {"Float32",NC_FLOAT,sizeof(float)},
@@ -81,6 +83,7 @@ static const struct ATOMICTYPEINFO {
 {"UInt32",NC_UINT,sizeof(unsigned int)},
 {"UInt64",NC_UINT64,sizeof(unsigned long long)},
 {"UInt8",NC_UBYTE,sizeof(unsigned char)},
+{"Url",NC_STRING,sizeof(char*)},
 {NULL,NC_NAT,0}
 };
 
@@ -105,7 +108,7 @@ extern const char** ezxml_all_attr(ezxml_t xml, int* countp);
 /* Forwards */
 
 static int addOrigType(NCD4parser*, NCD4node* src, NCD4node* dst, const char* tag);
-static int defineAtomicTypes(NCD4parser*);
+static int defineAtomicTypes(NCD4meta*,NClist*);
 static void classify(NCD4node* container, NCD4node* node);
 static int convertString(union ATOMICS*, NCD4node* type, const char* s);
 static int downConvert(union ATOMICS*, NCD4node* type);
@@ -114,12 +117,13 @@ static NCD4node* getOpaque(NCD4parser*, ezxml_t varxml, NCD4node* group);
 static int getValueStrings(NCD4parser*, NCD4node*, ezxml_t xattr, NClist*);
 static int isReserved(const char* name);
 static const KEYWORDINFO* keyword(const char* name);
-static NCD4node* lookupAtomictype(NCD4parser*, const char* name);
+static NCD4node* lookupAtomicType(NClist*, const char* name);
 static NCD4node* lookFor(NClist* elems, const char* name, NCD4sort sort);
 static NCD4node* lookupFQN(NCD4parser*, const char* sfqn, NCD4sort);
 static int lookupFQNList(NCD4parser*, NClist* fqn, NCD4sort sort, NCD4node** result);
 static NCD4node* makeAnonDim(NCD4parser*, const char* sizestr);
 static int makeNode(NCD4parser*, NCD4node* parent, ezxml_t, NCD4sort, nc_type, NCD4node**);
+static int makeNodeStatic(NCD4meta* meta, NCD4node* parent, NCD4sort sort, nc_type subsort, NCD4node** nodep);
 static int parseAtomicVar(NCD4parser*, NCD4node* container, ezxml_t xml, NCD4node**);
 static int parseAttributes(NCD4parser*, NCD4node* container, ezxml_t xml);
 static int parseDimensions(NCD4parser*, NCD4node* group, ezxml_t xml);
@@ -140,8 +144,9 @@ static int parseVariable(NCD4parser*, NCD4node* group, ezxml_t xml, NCD4node**);
 static void reclaimParser(NCD4parser* parser);
 static void record(NCD4parser*, NCD4node* node);
 static int splitOrigType(NCD4parser*, const char* fqn, NCD4node* var);
-static void track(NCD4parser*, NCD4node* node);
+static void track(NCD4meta*, NCD4node* node);
 static int traverse(NCD4parser*, ezxml_t dom);
+static int parseForwards(NCD4parser* parser, NCD4node* root);
 #ifndef FIXEDOPAQUE
 static int defineBytestringType(NCD4parser*);
 #endif
@@ -157,6 +162,10 @@ NCD4_parse(NCD4meta* metadata)
     int ilen;
     ezxml_t dom = NULL;
 
+    /* fill in the atomic types for meta*/
+    metadata->atomictypes = nclistnew();
+    if((ret=defineAtomicTypes(metadata,metadata->atomictypes))) goto done;
+
     /* Create and fill in the parser state */
     parser = (NCD4parser*)calloc(1,sizeof(NCD4parser));
     if(parser == NULL) {ret=NC_ENOMEM; goto done;}
@@ -171,7 +180,7 @@ NCD4_parse(NCD4meta* metadata)
     parser->debuglevel = 1;
 #endif
 
-    /*Walk the DOM tree */
+    /*Walk the DOM tree to build the DAP4 node tree*/
     ret = traverse(parser,dom);
 
 done:
@@ -184,19 +193,11 @@ done:
 static void
 reclaimParser(NCD4parser* parser)
 {
-    int i,len;
     if(parser == NULL) return;
     nclistfree(parser->types);
     nclistfree(parser->dims);
     nclistfree(parser->vars);
-    /* Reclaim unused atomic type nodes */
-    len = nclistlength(parser->atomictypes);    
-    for(i=0;i<len;i++) {
-	if(parser->used[i])
-	    reclaimNode((NCD4node*)nclistget(parser->atomictypes,i));
-    }
-    nclistfree(parser->atomictypes);
-    nullfree(parser->used);
+    nclistfree(parser->groups);
     free (parser);
 }
 
@@ -211,6 +212,12 @@ traverse(NCD4parser* parser, ezxml_t dom)
     /* See if we have an <Error> or <Dataset> */
     if(strcmp(dom->name,"Error")==0) {
 	ret=parseError(parser,dom);
+        /* Report the error */
+	fprintf(stderr,"DAP4 Error: http-code=%d message=\"%s\" context=\"%s\"\n",
+		parser->metadata->error.httpcode,
+		parser->metadata->error.message,
+		parser->metadata->error.context);
+	fflush(stderr);
 	ret=NC_EDMR;
 	goto done;
     } else if(strcmp(dom->name,"Dataset")==0) {
@@ -226,10 +233,14 @@ traverse(NCD4parser* parser, ezxml_t dom)
 	if(xattr != NULL) parser->metadata->root->group.dapversion = strdup(xattr);
 	xattr = ezxml_attr(dom,"dmrVersion");
 	if(xattr != NULL) parser->metadata->root->group.dmrversion = strdup(xattr);
-        /* fill in the atomic types */
-        if((ret=defineAtomicTypes(parser))) goto done;
         /* Recursively walk the tree */
         if((ret = fillgroup(parser,parser->metadata->root,dom))) goto done;
+
+        /* Walk a second time to parse allowed forward refs:
+	   1. <Map>
+	*/
+        if((ret = parseForwards(parser,parser->metadata->root))) goto done;
+
     } else
 	FAIL(NC_EINVAL,"Unexpected dom root name: %s",dom->name);
 done:
@@ -375,8 +386,9 @@ parseVariable(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
     default:
 	ret = parseAtomicVar(parser,container,xml,&node);
     }
-    *nodep = node;
-
+    if(ret == NC_NOERR) {
+        *nodep = node;
+    }
     return THROW(ret);
 }
 
@@ -426,8 +438,6 @@ parseStructure(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node**
     /* Parse attributes, dims, and maps into the var */
     if((ret = parseMetaData(parser,var,xml))) goto done;
 
-    record(parser,var);
-
     /* See if this var has UCARTAGORIGTYPE attribute */
     if(parser->metadata->controller->controls.translation == NCD4_TRANSNC4) {
 	const char* typetag = ezxml_attr(xml,UCARTAGORIGTYPE);
@@ -535,7 +545,7 @@ parseSequence(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
 	vlentype->basetype = var->basetype;
 	/* Use name <fqnname>_t */
 	strncpy(name,fqnname,sizeof(name));
-	strncat(name,"_t", sizeof(name) - strlen(name) - 1);
+	strlcat(name,"_t", sizeof(name));
         SETNAME(vlentype,name);
         /* Set the basetype */
         var->basetype = vlentype;
@@ -550,7 +560,7 @@ parseSequence(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
         classify(group,structtype);
 	/* Use name <fqnname>_base */
 	strncpy(name,fqnname,sizeof(name));
-	strncat(name,"_base", sizeof(name) - strlen(name) - 1);
+	strlcat(name,"_base", sizeof(name));
         SETNAME(structtype,name);
         /* Parse Fields into type */
         if((ret = parseFields(parser,structtype,xml))) goto done;
@@ -559,7 +569,7 @@ parseSequence(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
         classify(group,vlentype);
 	/* Use name <xname>_t */
 	strncpy(name,fqnname,sizeof(name));
-	strncat(name,"_t", sizeof(name) - strlen(name) - 1);
+	strlcat(name,"_t", sizeof(name));
         SETNAME(vlentype,name);
 	vlentype->basetype = structtype;
         /* Set the basetype */
@@ -569,8 +579,6 @@ parseSequence(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
     /* Parse attributes, dims, and maps into var*/
     if((ret = parseMetaData(parser,var,xml))) goto done;
 
-    record(parser,var);
-
     /* See if this var has UCARTAGORIGTYPE attribute */
     if(parser->metadata->controller->controls.translation == NCD4_TRANSNC4) {
 	const char* typetag = ezxml_attr(xml,UCARTAGORIGTYPE);
@@ -597,10 +605,10 @@ parseGroups(NCD4parser* parser, NCD4node* parent, ezxml_t xml)
 	if(name == NULL) FAIL(NC_EBADNAME,"Group has no name");
 	if((ret=makeNode(parser,parent,x,NCD4_GROUP,NC_NULL,&group))) goto done;
 	group->group.varbyid = nclistnew();
+	classify(parent,group);
         if((ret = fillgroup(parser,group,x))) goto done;
         /* Parse group attributes */
         if((ret = parseAttributes(parser,group,x))) goto done;
-	PUSH(parent->groups,group);
     }
 done:
     return THROW(ret);
@@ -695,15 +703,11 @@ parseMaps(NCD4parser* parser, NCD4node* var, ezxml_t xml)
     ezxml_t x;
 
     for(x=ezxml_child(xml, "Map");x!= NULL;x=ezxml_next(x)) {
-	NCD4node* mapref = NULL;
 	const char* fqn;
 	fqn = ezxml_attr(x,"name");
 	if(fqn == NULL)
 	    FAIL(NC_ENOTVAR,"<Map> has no name attribute");
-        mapref = lookupFQN(parser,fqn,NCD4_VAR);
-	if(mapref == NULL)
-	    FAIL(NC_ENOTVAR,"<Map> name does not refer to a variable: %s",fqn);
-	PUSH(var->maps,mapref);
+	PUSH(var->mapnames,strdup(fqn));
     }
 done:
     return THROW(ret);
@@ -726,8 +730,8 @@ parseAttributes(NCD4parser* parser, NCD4node* container, ezxml_t xml)
 	    container->xmlattributes = nclistnew();
 	    for(p=all;*p;p+=2) {
 		if(isReserved(*p)) {
-		    nclistpush(container->xmlattributes,strdup(p[0]));
-		    nclistpush(container->xmlattributes,strdup(p[1]));
+		    PUSH(container->xmlattributes,strdup(p[0]));
+		    PUSH(container->xmlattributes,strdup(p[1]));
 		}
 	    }
 	}
@@ -754,9 +758,9 @@ parseAttributes(NCD4parser* parser, NCD4node* container, ezxml_t xml)
 	if((ret=makeNode(parser,container,x,NCD4_ATTR,NC_NULL,&attr))) goto done;
 	basetype = lookupFQN(parser,type,NCD4_TYPE);
 	if(basetype == NULL)
-	    FAIL(NC_EBADTYPE,"Unknown <Attribute> type: ",type);
+	    FAIL(NC_EBADTYPE,"Unknown <Attribute> type: %s",type);
 	if(basetype->subsort == NC_NAT && basetype->subsort != NC_ENUM)
-	    FAIL(NC_EBADTYPE,"<Attribute> type must be atomic or enum: ",type);
+	    FAIL(NC_EBADTYPE,"<Attribute> type must be atomic or enum: %s",type);
 	attr->basetype = basetype;
 	values = nclistnew();
 	if((ret=getValueStrings(parser,basetype,x,values))) {
@@ -849,8 +853,6 @@ getOpaque(NCD4parser* parser, ezxml_t varxml, NCD4node* group)
 	        goto done;
   	    SETNAME(opaquetype,name);
 	    opaquetype->opaque.size = len;
-	    if(opaquetype != NULL)
-	        record(parser,opaquetype);
 	}
     }
 done:
@@ -882,7 +884,7 @@ getValueStrings(NCD4parser* parser, NCD4node* type, ezxml_t xattr, NClist* svalu
 	    /* Need to de-escape the string */
 	    es = NCD4_entityescape(s);
 	    ds = NCD4_deescape(es);
-	    nclistpush(svalues,ds);
+	    PUSH(svalues,ds);
 	    nullfree(es);
 	}
     }
@@ -933,7 +935,7 @@ splitOrigType(NCD4parser* parser, const char* fqn, NCD4node* type)
     /* It should be the case that the pieces are {/group}+/name */
     name = (char*)nclistpop(pieces);
     if((ret = lookupFQNList(parser,pieces,NCD4_GROUP,&group))) goto done;
-    if(group == NULL) {
+    if(ret) {
 	FAIL(NC_ENOGRP,"Non-existent group in FQN: ",fqn);
     }
     type->nc4.orig.name = strdup(name+1); /* plus 1 to skip the leading separator */
@@ -1015,6 +1017,8 @@ WARNING: This is highly specialized in that it assumes
 that the final object is one of: dimension, type, or var.
 This means that e.g. groups, attributes, econsts, cannot
 be found by this procedure.
+@return NC_NOERR found; result != NULL
+@return NC_EINVAL !found; result == NULL
 */
 static int
 lookupFQNList(NCD4parser* parser, NClist* fqn, NCD4sort sort, NCD4node** result)
@@ -1044,20 +1048,20 @@ lookupFQNList(NCD4parser* parser, NClist* fqn, NCD4sort sort, NCD4node** result)
 	3. i < (nsteps - 1) => need a compound var to continue
     */
     if(i == nsteps) {
-	if(sort != NCD4_GROUP) goto sortfail;
-	goto done;
+	if(sort != NCD4_GROUP) /* Right name, wrong sort */
+	goto notfound;
     }
     if(i == (nsteps - 1)) {
 	assert (node == NULL);
         node = lookFor(current->group.elements,name,sort);
-	if(node == NULL) goto sortfail;
+	if(node == NULL) goto notfound;
 	goto done;
     }
     assert (i < (nsteps - 1)); /* case 3 */
     /* We have steps to take, so node better be a compound var */
     node = lookFor(current->group.elements,name,NCD4_VAR);
     if(node == NULL || !ISCMPD(node->basetype->subsort))
-	goto fail;
+	goto notfound;
     /* So we are at a compound variable, so walk its fields recursively */
     /* Use the type to do the walk */
     current = node->basetype;
@@ -1073,22 +1077,19 @@ lookupFQNList(NCD4parser* parser, NClist* fqn, NCD4sort sort, NCD4node** result)
 		{node = field; break;}
 	}
 	if(node == NULL)
-	    goto sortfail; /* no match, so failed */
+	    goto notfound; /* no match, so failed */
 	if(i == (nsteps - 1))
 	    break;
 	if(!ISCMPD(node->basetype->subsort))
-	    goto fail; /* more steps, but no compound field, so failed */
+	    goto notfound; /* more steps, but no compound field, so failed */
 	current = node->basetype;
     }
 done:
     if(result) *result = node;
     return THROW(ret);
-fail:
+notfound:
     ret = NC_EINVAL;
     goto done;
-sortfail:
-    ret = NC_EBADID;
-    goto done;
 }
 
 static NCD4node*
@@ -1130,7 +1131,7 @@ lookupFQN(NCD4parser* parser, const char* sfqn, NCD4sort sort)
 
     /* Short circuit atomic types */
     if(NCD4_TYPE == sort) {
-        match = lookupAtomictype(parser,(sfqn[0]=='/'?sfqn+1:sfqn));
+        match = lookupAtomicType(parser->metadata->atomictypes,(sfqn[0]=='/'?sfqn+1:sfqn));
         if(match != NULL)
 	    goto done;
     }
@@ -1175,7 +1176,7 @@ defineBytestringType(NCD4parser* parser)
         if(ret != NC_NOERR) goto done;
         SETNAME(bstring,"_bytestring");
 	bstring->opaque.size = 0;
-	bstring->basetype = lookupAtomictype(parser,"UInt8");
+	bstring->basetype = lookupAtomicType(parser,"UInt8");
         PUSH(parser->metadata->root->types,bstring);
 	parser->metadata->_bytestring = bstring;
     } else
@@ -1186,34 +1187,28 @@ done:
 #endif
 
 static int
-defineAtomicTypes(NCD4parser* parser)
+defineAtomicTypes(NCD4meta* meta, NClist* list)
 {
     int ret = NC_NOERR;
     NCD4node* node;
     const struct ATOMICTYPEINFO* ati;
-
-    parser->atomictypes = nclistnew();
-    if(parser->atomictypes == NULL)
-	return THROW(NC_ENOMEM);
+ 
+    if(list == NULL)
+	return THROW(NC_EINTERNAL);
     for(ati=atomictypeinfo;ati->name;ati++) {
-        if((ret=makeNode(parser,parser->metadata->root,NULL,NCD4_TYPE,ati->type,&node))) goto done;
+        if((ret=makeNodeStatic(meta,NULL,NCD4_TYPE,ati->type,&node))) goto done;
 	SETNAME(node,ati->name);
-        node->container = parser->metadata->root;
-	record(parser,node);
-	PUSH(parser->atomictypes,node);
+	PUSH(list,node);
     }
-    parser->used = (char*)calloc(1,nclistlength(parser->atomictypes));
-    if(parser->used == NULL) {ret = NC_ENOMEM; goto done;}
-
 done:
     return THROW(ret);
 }
 
 /* Binary search the set of set of atomictypes */
 static NCD4node*
-lookupAtomictype(NCD4parser* parser, const char* name)
+lookupAtomicType(NClist* atomictypes, const char* name)
 {
-    int n = nclistlength(parser->atomictypes);
+    int n = nclistlength(atomictypes);
     int L = 0;
     int R = (n - 1);
     int m, cmp;
@@ -1222,9 +1217,10 @@ lookupAtomictype(NCD4parser* parser, const char* name)
     for(;;) {
 	if(L > R) break;
         m = (L + R) / 2;
-	p = (NCD4node*)nclistget(parser->atomictypes,m);
+	p = (NCD4node*)nclistget(atomictypes,m);
 	cmp = strcasecmp(p->name,name);
-	if(cmp == 0) return p;
+	if(cmp == 0)
+	    return p;
 	if(cmp < 0)
 	    L = (m + 1);
 	else /*cmp > 0*/
@@ -1238,13 +1234,12 @@ lookupAtomictype(NCD4parser* parser, const char* name)
 static int
 makeNode(NCD4parser* parser, NCD4node* parent, ezxml_t xml, NCD4sort sort, nc_type subsort, NCD4node** nodep)
 {
-    int ret = NC_NOERR;
-    NCD4node* node = (NCD4node*)calloc(1,sizeof(NCD4node));
+    int ret = NC_NOERR;    
+    NCD4node* node = NULL;
+    
+    assert(parser);
+    if((ret = makeNodeStatic(parser->metadata,parent,sort,subsort,&node))) goto done;
 
-    if(node == NULL) return THROW(NC_ENOMEM);
-    node->sort = sort;
-    node->subsort = subsort;
-    node->container = parent;
     /* Set node name, if it exists */
     if(xml != NULL) {
         const char* name = ezxml_attr(xml,"name");
@@ -1255,11 +1250,30 @@ makeNode(NCD4parser* parser, NCD4node* parent, ezxml_t xml, NCD4sort sort, nc_ty
 	    SETNAME(node,name);
 	}
     }
+    record(parser,node);
+    if(nodep) *nodep = node;
+done:
+    return ret;
+}
+
+static int
+makeNodeStatic(NCD4meta* meta, NCD4node* parent, NCD4sort sort, nc_type subsort, NCD4node** nodep)
+{
+    int ret = NC_NOERR;
+    NCD4node* node = (NCD4node*)calloc(1,sizeof(NCD4node));
+
+    assert(meta);
+    if(node == NULL) return THROW(NC_ENOMEM);
+    node->sort = sort;
+    node->subsort = subsort;
+    node->container = parent;
+#if 0
     if(parent != NULL) {
 	if(parent->sort == NCD4_GROUP)
-	    PUSH(parent->group.elements,node);
+	    classify(parent,node);
     }
-    track(parser,node);
+#endif
+    track(meta,node);
     if(nodep) *nodep = node;
     return THROW(ret);
 }
@@ -1283,7 +1297,7 @@ makeAnonDim(NCD4parser* parser, const char* sizestr)
 	SETNAME(dim,name+1); /* leave out the '/' separator */
 	dim->dim.size = (long long)size;
 	dim->dim.isanonymous = 1;
-	PUSH(root->dims,dim);
+	classify(root,dim);
     }
 done:
     return (ret?NULL:dim);
@@ -1297,7 +1311,7 @@ static void
 classify(NCD4node* container, NCD4node* node)
 {
     if(ISGROUP(container->sort))
-	nclistpush(container->group.elements,node);
+	PUSH(container->group.elements,node);
     switch (node->sort) {
     case NCD4_GROUP:
 	PUSH(container->groups,node);
@@ -1374,7 +1388,7 @@ forget(NCD4parser* parser, NCD4node* var)
 #endif
 
 static void
-track(NCD4parser* parser, NCD4node* node)
+track(NCD4meta* meta, NCD4node* node)
 {
 #ifdef D4DEBUG
     fprintf(stderr,"track: node=%lx sort=%d subsort=%d",(unsigned long)node,node->sort,node->subsort);
@@ -1382,9 +1396,9 @@ track(NCD4parser* parser, NCD4node* node)
         fprintf(stderr," name=%s\n",node->name);
     fprintf(stderr,"\n");
 #endif
-    PUSH(parser->metadata->allnodes,node);
+    PUSH(meta->allnodes,node);
 #ifdef D4DEBUG
-    fprintf(stderr,"track: |allnodes|=%ld\n",nclistlength(parser->metadata->allnodes));
+    fprintf(stderr,"track: |allnodes|=%ld\n",nclistlength(meta->allnodes));
     fflush(stderr);
 #endif
 }
@@ -1495,7 +1509,7 @@ valueParse(NCD4node* type, const char* values0, NClist* vlist)
 	    q = p - 1;
 	    *p++ = '\0';
 	    if(*q == '\r') {*q = '\0';}
-            nclistpush(vlist,strdup(line));
+            PUSH(vlist,strdup(line));
 	}
 	break;
     case NC_CHAR:
@@ -1513,7 +1527,7 @@ valueParse(NCD4node* type, const char* values0, NClist* vlist)
 	    if(len > 0) {
 		c[0] = *q;
 		c[1] = '\0';
-		nclistpush(vlist,strdup(c));
+		PUSH(vlist,strdup(c));
 	    }
 	}
 	break;
@@ -1525,7 +1539,7 @@ valueParse(NCD4node* type, const char* values0, NClist* vlist)
 	for(line=values;*line;) {
 	    size_t size = strlen(line);
 	    if(size > 0)
-	        nclistpush(vlist,strdup(line));
+	        PUSH(vlist,strdup(line));
 	    line += (size+1); /* skip terminating nul */
 	}
 	break;
@@ -1534,3 +1548,48 @@ valueParse(NCD4node* type, const char* values0, NClist* vlist)
     return THROW(NC_NOERR);
 }
 #endif
+
+/* Define extra attributes not present in the xml */
+int
+NCD4_defineattr(NCD4meta* meta, NCD4node* parent, const char* aname, const char* typename, NCD4node** attrp)
+{
+    NCD4node* attr = NULL;
+    NCD4node* basetype = NULL;
+    if((basetype = lookupAtomicType(meta->atomictypes,typename))==NULL)
+	return NC_EINVAL;
+    if(makeNode(NULL,parent,NULL,NCD4_ATTR,NC_NULL,&attr))
+	return NC_EINVAL;
+    SETNAME(attr,strdup(aname));
+    attr->basetype = basetype;
+    PUSH(parent->attributes,attr);		
+    if(attrp) *attrp = attr;
+    return NC_NOERR;
+}
+
+/*
+Fill in forward references for selected Node:
+1. <Map>
+*/
+static int
+parseForwards(NCD4parser* parser, NCD4node* root)
+{
+    int ret = NC_NOERR;
+    int i,j;
+
+    /* process all vars */
+    for(i=0;i<nclistlength(parser->vars);i++) {
+        NCD4node* var = (NCD4node*)nclistget(parser->vars,i);
+	/* Process the variable's maps */
+	for(j=0;j<nclistlength(var->mapnames);j++) {
+            const char* mapname = (const char*)nclistget(var->mapnames,j);
+            /* Find the corresponding variable */
+            NCD4node* mapref = lookupFQN(parser,mapname,NCD4_VAR);
+	    if(mapref == NULL)
+	        FAIL(NC_ENOTVAR,"<Map> name does not refer to a variable: %s",mapname);
+	    PUSH(var->maps,mapref);
+	}
+    }
+    
+done:
+    return THROW(ret);
+}
