@@ -6,16 +6,14 @@ Functions for inquiring about variables.
 */
 
 #include "config.h"
+#include "netcdf.h"
+#include "netcdf_filter.h"
 #include "ncdispatch.h"
+#include "nc4internal.h"
 #ifdef USE_HDF5
 #include <hdf5.h>
 #endif /* USE_HDF5 */
 
-#ifndef H5Z_FILTER_SZIP
-/** ID of HDF SZIP filter. */
-#define H5Z_FILTER_SZIP 4
-#endif
-
 /** \name Learning about Variables
 
 Functions to learn about the variables in a file. */
@@ -257,10 +255,18 @@ nc_inq_varnatts(int ncid, int varid, int *nattsp)
 		     nattsp);
 }
 
-/** \ingroup variables
-Learn the storage and deflate settings for a variable.
+/** 
+\ingroup variables 
 
-This is a wrapper for nc_inq_var_all().
+Learn the shuffle and deflate settings for a variable.
+
+Deflation is compression with the zlib library. Shuffle re-orders the
+data bytes to provide better compression (see nc_def_var_deflate()).
+
+Deflation is only available for HDF5 files. For classic and other
+files, this function will return setting that indicate that deflation
+is not in use, and that the shuffle filter is not in use. That is:
+shuffle off, deflate off, and a deflate level of 0.
 
 \param ncid NetCDF or group ID, from a previous call to nc_open(),
 nc_create(), nc_def_grp(), or associated inquiry functions such as
@@ -276,21 +282,58 @@ function will write a 1 if the deflate filter is turned on for this
 variable, and a 0 otherwise. \ref ignored_if_null.
 
 \param deflate_levelp If the deflate filter is in use for this
-variable, the deflate_level will be written here. \ref ignored_if_null.
+variable, the deflate_level will be written here. If deflate is not in
+use, and deflate_levelp is provided, it will get a zero. (This
+behavior is expected by the Fortran APIs). \ref ignored_if_null.
 
 \returns ::NC_NOERR No error.
-\returns ::NC_ENOTNC4 Not a netCDF-4 file.
 \returns ::NC_EBADID Bad ncid.
 \returns ::NC_ENOTVAR Invalid variable ID.
+\author Ed Hartnett, Dennis Heimbigner
 */
 int
-nc_inq_var_deflate(int ncid, int varid, int *shufflep, int *deflatep,
-		   int *deflate_levelp)
+nc_inq_var_deflate(int ncid, int varid, int *shufflep, int *deflatep, int *deflate_levelp)
 {
    NC* ncp;
-   int stat = NC_check_id(ncid,&ncp);
+   size_t nparams;
+   unsigned int params[4];
+   int deflating = 0;
+   int stat;
+   
+   stat = NC_check_id(ncid,&ncp);
    if(stat != NC_NOERR) return stat;
    TRACE(nc_inq_var_deflate);
+
+   /* Verify id and  nparams */
+   stat = nc_inq_var_filter_info(ncid,varid,H5Z_FILTER_DEFLATE,&nparams,params);
+   switch (stat) {
+   case NC_ENOFILTER: deflating = 0; stat = NC_NOERR; break;
+   case NC_NOERR: deflating = 1; break;
+   case NC_ENOTNC4:
+       /* As a special case, to support behavior already coded into user
+	* applications, handle classic format files by reporting no
+	* deflation. */
+       if (shufflep)
+	   *shufflep = 0;
+       if (deflatep)
+	   *deflatep = 0;
+       if (deflate_levelp)
+	   *deflate_levelp = 0;
+       return NC_NOERR;
+       break;
+   default: return stat;
+   }
+   if(deflatep) *deflatep = deflating;
+   if(deflating) {
+        if(nparams != 1)
+	    return NC_EFILTER; /* bad # params */
+	/* Param[0] should be level */
+	if(deflate_levelp) *deflate_levelp = (int)params[0];
+   } else if (deflate_levelp)
+       *deflate_levelp = 0;
+   /* also get the shuffle state */
+   if(!shufflep)
+       return NC_NOERR;
    return ncp->dispatch->inq_var_all(
       ncid, varid,
       NULL, /*name*/
@@ -299,15 +342,15 @@ nc_inq_var_deflate(int ncid, int varid, int *shufflep, int *deflatep,
       NULL, /*dimidsp*/
       NULL, /*nattsp*/
       shufflep, /*shufflep*/
-      deflatep, /*deflatep*/
-      deflate_levelp, /*deflatelevelp*/
+      NULL, /*deflatep*/
+      NULL, /*deflatelevelp*/
       NULL, /*fletcher32p*/
       NULL, /*contiguousp*/
       NULL, /*chunksizep*/
       NULL, /*nofillp*/
       NULL, /*fillvaluep*/
       NULL, /*endianp*/
-      NULL,NULL,NULL
+      NULL, NULL, NULL
       );
 }
 
@@ -358,32 +401,33 @@ nc_inq_var_fletcher32(int ncid, int varid, int *fletcher32p)
       );
 }
 
-/** \ingroup variables
-
-This is a wrapper for nc_inq_var_all().
-
-\param ncid NetCDF or group ID, from a previous call to nc_open(),
-nc_create(), nc_def_grp(), or associated inquiry functions such as
-nc_inq_ncid().
-
-\param varid Variable ID
-
-\param storagep Address of returned storage property, returned as
-::NC_CONTIGUOUS if this variable uses contiguous storage, or
-::NC_CHUNKED if it uses chunked storage. \ref ignored_if_null.
-
-\param chunksizesp The chunksizes will be copied here. \ref
-ignored_if_null.
-
-\returns ::NC_NOERR No error.
-\returns ::NC_EBADID Bad ncid.
-\returns ::NC_ENOTNC4 Not a netCDF-4 file.
-\returns ::NC_ENOTVAR Invalid variable ID.
-
-
-\section nc_inq_var_chunking_example Example
-
-\code
+/**
+ * @ingroup variables
+ *
+ * Get the storage and (for chunked variables) the chunksizes of a
+ * variable. See nc_def_var_chunking() for explanation of storage.
+ *
+ * @param ncid NetCDF or group ID, from a previous call to nc_open(),
+ * nc_create(), nc_def_grp(), or associated inquiry functions such as
+ * nc_inq_ncid().
+ * @param varid Variable ID
+ * @param storagep Address of returned storage property, returned as
+ * ::NC_CONTIGUOUS if this variable uses contiguous storage,
+ * ::NC_CHUNKED if it uses chunked storage, or ::NC_COMPACT for
+ * compact storage. \ref ignored_if_null.
+ * @param chunksizesp The chunksizes will be copied here. \ref
+ * ignored_if_null.
+ *
+ * @return ::NC_NOERR No error.
+ * @return ::NC_EBADID Bad ncid.
+ * @return ::NC_ENOTNC4 Not a netCDF-4 file.
+ * @return ::NC_ENOTVAR Invalid variable ID.
+ *
+ * @author Ed Hartnett
+ *
+ * @section nc_inq_var_chunking_example Example
+ *
+ * @code
         printf("**** testing contiguous storage...");
         {
      #define NDIMS6 1
@@ -415,8 +459,8 @@ ignored_if_null.
 
            if (nc_inq_var_chunking(ncid, 0, &storage_in, chunksize_in)) ERR;
            if (storage_in != NC_CONTIGUOUS) ERR;
-\endcode
-
+@endcode
+*
 */
 int
 nc_inq_var_chunking(int ncid, int varid, int *storagep, size_t *chunksizesp)
@@ -459,10 +503,12 @@ nc_inq_var_fill(int ncid, int varid, int *no_fill, void *fill_valuep)
 {
    NC* ncp;
    int stat = NC_check_id(ncid,&ncp);
+
    if(stat != NC_NOERR) return stat;
    TRACE(nc_inq_var_fill);
+
    return ncp->dispatch->inq_var_all(
-      ncid, varid,
+      ncid,varid,
       NULL, /*name*/
       NULL, /*xtypep*/
       NULL, /*ndimsp*/
@@ -608,67 +654,24 @@ nc_inq_unlimdims(int ncid, int *nunlimdimsp, int *unlimdimidsp)
 }
 
 /**
-Find the filter (if any) associated with a variable.
-
-This is a wrapper for nc_inq_var_all().
-
-\param ncid NetCDF or group ID, from a previous call to nc_open(),
-nc_create(), nc_def_grp(), or associated inquiry functions such as
-nc_inq_ncid().
-
-\param varid Variable ID
-
-\param idp Storage which will get the filter id.
-
-\param nparamsp Storage which will get the number of parameters to the
-filter
-
-\param params Storage which will get associated parameters. Note
-the caller must allocate and free.
-
-\returns ::NC_NOERR No error.
-\returns ::NC_ENOTNC4 Not a netCDF-4 file.
-\returns ::NC_EBADID Bad ncid.
-\returns ::NC_ENOTVAR Invalid variable ID.
-\returns ::NC_EFILTER No filter defined.
 \ingroup variables
-\author Dennis Heimbigner
-*/
-int
-nc_inq_var_filter(int ncid, int varid, unsigned int* idp, size_t* nparamsp, unsigned int* params)
-{
-   NC* ncp;
-   int stat = NC_check_id(ncid,&ncp);
-   if(stat != NC_NOERR) return stat;
-   TRACE(nc_inq_var_filter);
-   return ncp->dispatch->inq_var_all(
-      ncid, varid,
-      NULL, /*name*/
-      NULL, /*xtypep*/
-      NULL, /*ndimsp*/
-      NULL, /*dimidsp*/
-      NULL, /*nattsp*/
-      NULL, /*shufflep*/
-      NULL, /*deflatep*/
-      NULL, /*deflatelevelp*/
-      NULL, /*fletcher32p*/
-      NULL, /*contiguousp*/
-      NULL, /*chunksizep*/
-      NULL, /*nofillp*/
-      NULL, /*fillvaluep*/
-      NULL, /*endianp*/
-      idp, nparamsp, params);
-}
-
-/** \ingroup variables
 Learn the szip settings of a variable.
-Similar to nc_inq_var_deflate.
 
-This function returns the szip settings for a variable.
-With the introduction of general filter support,
-szip inquiry is converted to use the filter interface.
+This function returns the szip settings for a variable. To turn on
+szip compression, use nc_def_var_szip(). Szip compression is only
+available for netCDF/HDF5 files, and only if HDF5 was built with szip
+support. 
 
-This is a wrapper for nc_inq_var_filter().
+If a variable is not using szip, or if this function is called on a
+file that is not a HDF5 file, then a zero will be passed back for both
+options_maskp and pixels_per_blockp.
+
+For more information on HDF5 and szip see
+https://support.hdfgroup.org/HDF5/doc/RM/RM_H5P.html#Property-SetSzip
+and https://support.hdfgroup.org/doc_resource/SZIP/index.html.
+
+The nc_def_var_filter function may also be used to set szip
+compression.
 
 \param ncid NetCDF or group ID, from a previous call to nc_open(),
 nc_create(), nc_def_grp(), or associated inquiry functions such as
@@ -677,22 +680,28 @@ nc_inq_ncid().
 \param varid Variable ID
 
 \param options_maskp The szip options mask will be copied to this
-pointer. \ref ignored_if_null.
+pointer. Note that the HDF5 layer adds to the options_mask, so this
+value may be different from the value used when setting szip
+compression, however the bit set when setting szip compression will
+still be set in the options_maskp and can be checked for. If zero is
+returned, szip is not in use for this variable.\ref ignored_if_null.
 
 \param pixels_per_blockp The szip pixels per block will be copied
-here. \ref ignored_if_null.
+here. The HDF5 layer may change this value, so this may not match the
+value passed in when setting szip compression. If zero is returned,
+szip is not in use for this variable. \ref ignored_if_null.
 
 \returns ::NC_NOERR No error.
 \returns ::NC_EBADID Bad ncid.
-\returns ::NC_ENOTNC4 Not a netCDF-4 file.
 \returns ::NC_ENOTVAR Invalid variable ID.
-\returns ::NC_EFILTER Variable is not szip encoded
+\returns ::NC_EFILTER Filter error.
+
+\author Ed Hartnett, Dennis Heimbigner
 */
 int
 nc_inq_var_szip(int ncid, int varid, int *options_maskp, int *pixels_per_blockp)
 {
    NC* ncp;
-   unsigned int id;
    size_t nparams;
    unsigned int params[4];
 
@@ -700,128 +709,30 @@ nc_inq_var_szip(int ncid, int varid, int *options_maskp, int *pixels_per_blockp)
    if(stat != NC_NOERR) return stat;
    TRACE(nc_inq_var_szip);
 
-   /* Verify id and  nparams */
-   stat = ncp->dispatch->inq_var_all(
-      ncid, varid,
-      NULL, /*name*/
-      NULL, /*xtypep*/
-      NULL, /*ndimsp*/
-      NULL, /*dimidsp*/
-      NULL, /*nattsp*/
-      NULL, /*shufflep*/
-      NULL, /*deflatep*/
-      NULL, /*deflatelevelp*/
-      NULL, /*fletcher32p*/
-      NULL, /*contiguousp*/
-      NULL, /*chunksizep*/
-      NULL, /*nofillp*/
-      NULL, /*fillvaluep*/
-      NULL, /*endianp*/
-      &id,
-      &nparams,
-      NULL
-      );
-   if(stat != NC_NOERR) return stat;
-   /* Warning: the szip filter internally expands the set of parameters */
-   if(id != H5Z_FILTER_SZIP || nparams != 4)
-	return NC_EFILTER; /* not szip or bad # params */
-   /* Get params */
-   stat = ncp->dispatch->inq_var_all(
-      ncid, varid,
-      NULL, /*name*/
-      NULL, /*xtypep*/
-      NULL, /*ndimsp*/
-      NULL, /*dimidsp*/
-      NULL, /*nattsp*/
-      NULL, /*shufflep*/
-      NULL, /*deflatep*/
-      NULL, /*deflatelevelp*/
-      NULL, /*fletcher32p*/
-      NULL, /*contiguousp*/
-      NULL, /*chunksizep*/
-      NULL, /*nofillp*/
-      NULL, /*fillvaluep*/
-      NULL, /*endianp*/
-      &id,
-      &nparams,
-      params
-      );
-   if(stat != NC_NOERR) return stat;
-   /* Param[0] should be options_mask with possibly some other flags set,
+   /* Verify id and nparams */
+   stat = nc_inq_var_filter_info(ncid,varid,H5Z_FILTER_SZIP,&nparams,params);
+   switch (stat) {
+   case NC_NOERR:
+        if(nparams != 2)
+	    return NC_EFILTER; /* bad # params */
+	break;
+   case NC_ENOFILTER:
+   case NC_ENOTNC4:
+       /* If the szip filter is not in use, or if this is not a HDF5
+	* file, return 0 for both parameters. */
+       params[0] = 0;
+       params[1] = 0;
+       stat = NC_NOERR;
+       break;	   
+   default:
+   	return stat;
+   }
+
+   /* Param[0] should be options_mask
       Param[1] should be pixels_per_block */
    if(options_maskp) *options_maskp = (int)params[0];
    if(pixels_per_blockp) *pixels_per_blockp = (int)params[1];
-   return NC_NOERR;
-}
-
-/*!
-Learn all about a variable.
-
-@param[in] ncid ncid for file.
-@param[in] varid varid for variable in question.
-@param[out] name Pointer to memory to contain the name of the
-variable.
-@param[out] xtypep Pointer to memory to contain the type of the
-variable.
-@param[out] ndimsp Pointer to memory to store the number of associated
-dimensions for the variable.
-@param[out] dimidsp Pointer to memory to store the dimids associated
-with the variable.
-@param[out] nattsp Pointer to memory to store the number of attributes
-associated with the variable.
-@param[out] shufflep Pointer to memory to store shuffle information
-associated with the variable.
-@param[out] deflatep Pointer to memory to store compression type
-associated with the variable.
-@param[out] deflate_levelp Pointer to memory to store compression
-level associated with the variable.
-@param[out] fletcher32p Pointer to memory to store compression
-information associated with the variable.
-@param[out] contiguousp Pointer to memory to store contiguous-data
-information associated with the variable.
-@param[out] chunksizesp Pointer to memory to store chunksize
-information associated with the variable.
-@param[out] no_fill Pointer to memory to store whether or not there is
-a fill value associated with the variable.
-@param[out] fill_valuep Pointer to memory to store the fill value (if
-one exists) for the variable.
-@param[out] endiannessp Pointer to memory to store endianness
-value. One of ::NC_ENDIAN_BIG ::NC_ENDIAN_LITTLE ::NC_ENDIAN_NATIVE
-@param[out] idp Pointer to memory to store filter id.
-@param[out] nparamsp Pointer to memory to store filter parameter count.
-@param[out] params Pointer to vector of unsigned integers into which
-to store filter parameters.
-\note Expose access to nc_inq_var_all().
-
-\returns ::NC_NOERR No error.
-\returns ::NC_EBADID Bad ncid.
-\returns ::NC_ENOTVAR Bad varid.
-\returns ::NC_ENOMEM Out of memory.
-\returns ::NC_EINVAL Invalid input.
-\author Ed Hartnett, Dennis Heimbigner
-\internal
-\ingroup variables
-*/
-int
-NC_inq_var_all(int ncid, int varid, char *name, nc_type *xtypep,
-               int *ndimsp, int *dimidsp, int *nattsp,
-               int *shufflep, int *deflatep, int *deflate_levelp,
-               int *fletcher32p, int *contiguousp, size_t *chunksizesp,
-               int *no_fill, void *fill_valuep, int *endiannessp,
-	       unsigned int* idp, size_t* nparamsp, unsigned int* params
-               )
-{
-   NC* ncp;
-   int stat = NC_check_id(ncid,&ncp);
-   if(stat != NC_NOERR) return stat;
-   return ncp->dispatch->inq_var_all(
-      ncid, varid, name, xtypep,
-      ndimsp, dimidsp, nattsp,
-      shufflep, deflatep, deflate_levelp, fletcher32p,
-      contiguousp, chunksizesp,
-      no_fill, fill_valuep,
-      endiannessp,
-      idp,nparamsp,params);
+   return stat;
 }
 
 /*! \} */  /* End of named group ...*/
