@@ -1,5 +1,5 @@
 /* This is part of the netCDF package.
-   Copyright 2005 University Corporation for Atmospheric Research/Unidata
+   Copyright 2018 University Corporation for Atmospheric Research/Unidata
    See COPYRIGHT file for conditions of use.
 
    Test netcdf-4 group code.
@@ -41,7 +41,7 @@ main(int argc, char **argv)
       int henry_vii_id;
       int henry_viii_id;
       int grpid_in[MAX_SIBLING_GROUPS], varids_in[MAX_SIBLING_GROUPS];
-      int dimids_in[MAX_SIBLING_GROUPS], nvars_in, ndims_in, ncid_in;
+      int nvars_in, ncid_in;
       int parent_ncid;
       char name_out[NC_MAX_NAME + 1];
       int num_grps;
@@ -114,17 +114,17 @@ main(int argc, char **argv)
       /* Create a classic model file. No groups will be allowed. */
       if (nc_create(FILE_NAME_CLASSIC_MODEL, NC_NETCDF4|NC_CLASSIC_MODEL, &ncid2)) ERR;
       if (nc_def_grp(ncid2, HENRY_VII, &henry_vii_id) != NC_ESTRICTNC3) ERR;
-      if (nc_def_var(ncid2, HENRY_IV, NC_INT, 0, NULL, NULL));
+      if (nc_def_var(ncid2, HENRY_IV, NC_INT, 0, NULL, NULL)) ERR;
       if (nc_close(ncid2)) ERR;
 
       /* Create a file with one group, a group to contain data about
        * Henry VII. */
       if (nc_create(FILE_NAME, NC_NETCDF4, &ncid)) ERR;
-      if (nc_def_var(ncid, HENRY_IV, NC_INT, 0, NULL, NULL));      
+      if (nc_def_var(ncid, HENRY_IV, NC_INT, 0, NULL, NULL)) ERR;
 
       /* Turn off define mode. It will automatically be turned back on
        * when nc_def_grp is called. */
-      if (nc_enddef(ncid)) ERR; 
+      if (nc_enddef(ncid)) ERR;
       if (nc_def_grp(ncid, HENRY_VII, &henry_vii_id)) ERR;
 
       /* Check it out. */
@@ -215,6 +215,7 @@ main(int argc, char **argv)
       if (nc_inq_ncid(ncid, HENRY_VII, &grpid_in)) ERR;
 
       /* Ensure that dimensions in parent are visible and correct. */
+      memset(dimids_in,0,sizeof(dimids_in));
       if (nc_inq_dimids(grpid_in, &ndims_in, dimids_in, 1)) ERR;
       if (ndims_in != NDIMS_IN_FILE || dimids_in[0] != dimid1 || dimids_in[1] != dimid2) ERR;
       if (nc_inq_dim(grpid_in, dimids_in[0], name_in, &len_in)) ERR;
@@ -376,7 +377,7 @@ main(int argc, char **argv)
       if (nc_inq_grps(ncid, NULL, grpid_in)) ERR;
       if (nc_inq_grpname(grpid_in[0], name_in)) ERR;
       if (strcmp(name_in, HENRY_VII)) ERR;
-      if (nc_inq_grpname(grpid_in[0], NULL)) ERR;      
+      if (nc_inq_grpname(grpid_in[0], NULL)) ERR;
       if (grpid_in[0] != grp_ncid) ERR;
       strcat(full_name, HENRY_VII);
       if (nc_inq_grpname_full(grpid_in[0], &len, full_name_in)) ERR;
@@ -442,7 +443,7 @@ main(int argc, char **argv)
       if (nc_inq_grpname_full(grpid_in[0], &len1, NULL)) ERR;
       if (len1 != strlen(full_name)) ERR;
       if (nc_inq_grpname_full(grpid_in[0], &len, full_name_in1)) ERR;
-      if (strcmp(full_name_in1, full_name)) ERR;      
+      if (strcmp(full_name_in1, full_name)) ERR;
 
       if (nc_close(ncid)) ERR;
 
