@@ -1,15 +1,17 @@
 /*********************************************************************
- *   Copyright 2011, University Corporation for Atmospheric Research
+ *   Copyright 2018, University Corporation for Atmospheric Research
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *   
  *   Stuff that's common to both ncdump and nccopy
  *
  *********************************************************************/
+#ifndef _UTILS_H
+#define _UTILS_H
 
 #include "config.h"
 
-#ifndef STREQ
-#define	STREQ(a, b)	(*(a) == *(b) && strcmp((a), (b)) == 0)
+#ifndef NCSTREQ
+#define	NCSTREQ(a, b)	(*(a) == *(b) && strcmp((a), (b)) == 0)
 #endif
 
 /* Delimiter for separating netCDF groups in absolute pathnames, same as for HDF5 */
@@ -61,13 +63,25 @@ extern char *progname;		/* for error messages */
 extern "C" {
 #endif
 
-#define NC_CHECK(fncall) {int statnc=fncall;if(statnc!=NC_NOERR)check(statnc,__FILE__,__LINE__);}
+/* For NDEBUG builds, provide a version of NC_CHECK that does not
+ * include a file name. Including a file name causes heartache for the
+ * debian package builders. They already use NDEBUG to turn off the
+ * file names in asserts. */
+#ifdef NDEBUG
+#define NC_CHECK(fncall) {int ncstat=fncall;if(ncstat!=NC_NOERR)check(ncstat,"",__LINE__);}
+#else
+#define NC_CHECK(fncall) {int ncstat=fncall;if(ncstat!=NC_NOERR)check(ncstat,__FILE__,__LINE__);}
+#endif /* NDEBUG */
 
 /* Print error message to stderr and exit */
 extern void	error ( const char *fmt, ... );
 
 /* Check error on malloc and exit with message if out of memory */
 extern void*    emalloc ( size_t size );
+/* Ditto calloc */
+extern void*    ecalloc ( size_t size );
+/* Ditto realloc */
+extern void*    erealloc (void* p, size_t size );
 
 /* Check error return.  If bad, print error message and exit. */
 extern void check(int err, const char* file, const int line);
@@ -83,6 +97,12 @@ void print_name(const char *name);
  * names */
 extern int  nc_inq_dimid2(int ncid, const char *dimname, int *dimidp);
 
+/* Convert a full path name to a group to the specific groupid. */
+extern int  nc_inq_grpid2(int ncid, const char *grpname0, int *grpidp);
+
+/* Convert a full path name to a varid to the specific varid + grpid */
+extern int nc_inq_varid2(int ncid, const char *path0, int* varidp, int* grpidp);
+
 /* Test if variable is a record variable */
 extern int  isrecvar ( int ncid, int varid );
 
@@ -157,3 +177,6 @@ extern int getrootid(int grpid);
 #ifdef __cplusplus
 }
 #endif
+
+#endif /* _UTILS_H */
+
