@@ -1,10 +1,9 @@
 /*********************************************************************
- *   Copyright 2009, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
 #include "includes.h"
-#include "nc_iter.h"
 
 #ifdef ENABLE_F77
 
@@ -19,7 +18,7 @@ f77_charconstant(Generator* generator, Symbol* sym, Bytebuffer* codebuf, ...)
     /* Just transfer charbuf to codebuf */
     Bytebuffer* charbuf;
     va_list ap;
-    vastart(ap,codebuf);
+    va_start(ap,codebuf);
     charbuf = va_arg(ap, Bytebuffer*);
     va_end(ap);
     bbNull(charbuf);
