@@ -1,4 +1,4 @@
-/* Copyright 2009, UCAR/Unidata and OPeNDAP, Inc.
+/* Copyright 2018, UCAR/Unidata and OPeNDAP, Inc.
    See the COPYRIGHT file for more information.
 */
 
@@ -51,7 +51,7 @@ dap_datasetbody(DAPparsestate* state, Object name, Object decls)
            at the dataset level; simulate an errorbody response
         */
 	ocnodes_free(dups);
-        dap_parse_error(state,"Duplicate dataset field names: %s",name,dupname);
+        dap_parse_error(state,"Duplicate dataset field names: %s",(char*)name,dupname);
 	state->error = OC_ENAMEINUSE;
 	return (Object)NULL;
     }
