@@ -1,3 +1,4 @@
+#include "config.h"
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
@@ -38,8 +39,6 @@ will generate an error.
 static int paramcheck(size_t nparams, const unsigned int* params);
 static void mismatch(size_t i, const char* which);
 
-extern void NC_filterfix8(void* mem, int decode);
-
 const H5Z_class2_t H5Z_TEST[1] = {{
     H5Z_CLASS_T_VERS,                /* H5Z_class_t version */
     (H5Z_filter_t)(H5Z_FILTER_TEST), /* Filter id number */
@@ -81,6 +80,7 @@ that the parameters passed to the filter
 are correct. Specifically, that endian-ness
 is correct. As a filter, it is the identify
 function, passing input to output unchanged.
+It also prints out the size of each chunk.
 
 Test cases format:
 1.The first param is the test index i.e. which test to execute.
@@ -95,37 +95,102 @@ H5Z_filter_test(unsigned int flags, size_t cd_nelmts,
 {
     void* newbuf;
     unsigned int testcase = 0;
+    size_t size = 1024 * sizeof(float) * 2;
 
     if(cd_nelmts == 0)
 	goto fail;
 
     testcase = cd_values[0];
 
-    if(testcase == TC_ENDIAN) {
+    switch (testcase) {
+    case TC_ENDIAN:
 	if(!paramcheck(cd_nelmts,cd_values))
 	    goto fail;
+	break;
+    case TC_ODDSIZE:
+        /* Print out the chunk size */
+        fprintf(stderr,"nbytes = %lld chunk size = %lld\n",(long long)nbytes,(long long)*buf_size);
+        fflush(stderr);
+	break;
+    default: break;
     }
 
-    if (flags & H5Z_FLAG_REVERSE) {
+    if (flags & H5Z_FLAG_REVERSE) { /* Decompress */
 
-        /* Replace buffer */
-        newbuf = H5allocate_memory(*buf_size,0);
-        if(newbuf == NULL) abort();
-        memcpy(newbuf,*buf,*buf_size);
-	/* reclaim old buffer */
-	H5free_memory(*buf);
+        if(testcase == TC_EXPANDED) {
+	    int i;
+	    float* b = (float*)*buf;
+fprintf(stderr,"TC_EXPANDED: decompress: nbytes=%u buf_size=%u xdata[0..8]=|",(unsigned)nbytes,(unsigned)*buf_size);
+	    for(i=0;i<8;i++) {
+		fprintf(stderr," %u",(int)(b[1024+i]));
+	    }
+	    fprintf(stderr,"|\n");
+            /* Replace buffer */
+#ifdef HAVE_H5ALLOCATE_MEMORY
+            newbuf = H5allocate_memory(*buf_size,0);
+#else
+            newbuf = malloc(*buf_size);
+#endif
+            if(newbuf == NULL) abort();
+            memcpy(newbuf,*buf,*buf_size);
+	
+	} else {
+            /* Replace buffer */
+#ifdef HAVE_H5ALLOCATE_MEMORY
+            newbuf = H5allocate_memory(*buf_size,0);
+#else
+            newbuf = malloc(*buf_size);
+#endif
+            if(newbuf == NULL) abort();
+            memcpy(newbuf,*buf,*buf_size);
+	}
+	
+        /* reclaim old buffer */
+#ifdef HAVE_H5FREE_MEMORY
+        H5free_memory(*buf);
+#else
+        free(*buf);
+#endif
         *buf = newbuf;
 
-    } else {
+    } else { /* Compress */
+        if(testcase == TC_EXPANDED) {
+	    int i;
+	    float* b;
+#if 0	
+fprintf(stderr,"TC_EXPANDED: compress: nbytes=%u buf_size=%u size=%u\n",(unsigned)nbytes,(unsigned)*buf_size,(unsigned)size);
+#endif
+	    /* Replace buffer with one that is bigger than the chunk size */
+#ifdef HAVE_H5ALLOCATE_MEMORY
+            newbuf = H5allocate_memory(size,0);
+#else
+            newbuf = malloc(size);
+#endif
+            if(newbuf == NULL) abort();
+	    b = (float*)newbuf;
+	    for(i=0;i<1024*2;i++) {
+		b[i] = (float)(17+i);
+	    }
+            memcpy(newbuf,*buf,*buf_size);
+	    *buf_size = size;
+        } else  {
+            /* Replace buffer */
+#ifdef HAVE_H5ALLOCATE_MEMORY
+            newbuf = H5allocate_memory(*buf_size,0);
+#else
+            newbuf = malloc(*buf_size);
+#endif
+            if(newbuf == NULL) abort();
+            memcpy(newbuf,*buf,*buf_size);
+	}
 
-        /* Replace buffer */
-        newbuf = H5allocate_memory(*buf_size,0);
-        if(newbuf == NULL) abort();
-        memcpy(newbuf,*buf,*buf_size);
 	/* reclaim old buffer */
-	H5free_memory(*buf);
+#ifdef HAVE_H5FREE_MEMORY
+        H5free_memory(*buf);
+#else
+        free(*buf);
+#endif
         *buf = newbuf;
-
     }
 
     return *buf_size;
@@ -190,7 +255,7 @@ paramcheck(size_t nparams, const unsigned int* params)
         case 8: {/*double*/
             double x;
 	    memcpy(mem,&params[i],sizeof(mem));
-	    NC_filterfix8(mem,1); /* Fix up endianness */
+	    NC_h5filterspec_fix8(mem,1); /* Fix up endianness */
             x = *(double*)mem;
 	    dval = DBLVAL;
             i++; /* takes two parameters */
@@ -202,9 +267,9 @@ paramcheck(size_t nparams, const unsigned int* params)
         case 10: {/*signed long long*/
             signed long long x;
 	    memcpy(mem,&params[i],sizeof(mem));
-	    NC_filterfix8(mem,1); /* Fix up endianness */
+	    NC_h5filterspec_fix8(mem,1); /* Fix up endianness */
             x = *(signed long long*)mem;
-	    NC_filterfix8(&x,1); /* Fix up endianness */
+	    NC_h5filterspec_fix8(&x,1); /* Fix up endianness */
 	    lval = -9223372036854775807L;
             i++; /* takes two parameters */
             if(lval != x) {
@@ -215,7 +280,7 @@ paramcheck(size_t nparams, const unsigned int* params)
         case 12: {/*unsigned long long*/
             unsigned long long x;
 	    memcpy(mem,&params[i],sizeof(mem));
-	    NC_filterfix8(mem,1); /* Fix up endianness */
+	    NC_h5filterspec_fix8(mem,1); /* Fix up endianness */
             x = *(unsigned long long*)mem;
 	    lval = 18446744073709551615UL;
             i++; /* takes two parameters */
